#' Sparse Co-Inercia Analysis
#'
#' This function conduct sparse CIA for a given tuning parameter value.
#' @param X  n-by-p transformed data matrix of \code{X}, samples are rows and columns are features.
#' @param Y  n-by-q transformeddata matrix of \code{Y}, samples are rows and columns are features.
#' @param na  p-by-1 vector, initial values of loading vector of variables in \code{X}.
#' @param nb  q-by-1 vector, initial values of loading vector of variables in \code{Y}.
#' @param lambdas 2-by-1 vector, containing sparsity penalty parameter values for
#' the features space of \code{X} and \code{Y} each.
#' @return This function returns a list object.
#' \item{call}{the matched call.}
#' \item{res.opta}{the estimated loading vector of of transformated matrix of \code{X}.}
#' \item{res.optb}{the estimated loading vector of of transformated matrix of \code{Y}.}
#' @seealso see also \code{\link{oCIA}}, \code{\link{sCIA.cv}}.
#' @examples
#' library(pCIA)
#' data("demoData"); attach(demoData)
#' X <- setX[1:n,]; nX <- sweep(X, 2, apply(X, 2, mean))
#' Y <- setY[1:n,]; nY <- sweep(Y, 2, apply(Y, 2, mean))
#' result.cia <- oCIA(nX, nY, D, Qx, Qy)
#' result.scia <- fitscia(nX, nY, result.cia$res.opta[,1], result.cia$res.optb[,1], c(6, 6))
#' @export
fitscia <- function(X, Y, na, nb, lambdas, maxiter=100) {
  call <- match.call()
  p <- dim(X)[2]
  q <- dim(Y)[2]
  iter <- 1
  obj.old <- 1
  for (iter in 1:maxiter) {
    # fitting - a
    res.a <- glmnet::glmnet(x=diag(1,p), 
                            y=t(X)%*%Y%*%nb, 
                            intercept = FALSE, 
                            alpha = 1,
                            lambda = lambdas[1])
    res.opta <- as.matrix(res.a$beta)
    if (as.numeric(sqrt(t(res.opta) %*% res.opta)) != 0) {
      res.opta <- res.opta / as.numeric(sqrt(t(res.opta) %*% res.opta))
    } else {
      res.opta = matrix(0, p, 1)
    }
    # fitting - b
    res.b <- glmnet::glmnet(x=diag(1,q),
                            y=t(Y)%*%X%*%na, 
                            intercept = FALSE, 
                            alpha = 1,
                            lambda = lambdas[2])
    res.optb <-  as.matrix(res.b$beta)
    if (as.numeric(sqrt(t(res.optb) %*% res.optb)) != 0) {
      res.optb <- res.optb / as.numeric(sqrt(t(res.optb) %*% res.optb))
    } else {
      res.optb <- matrix(0, q, 1)
    }
    obj.new <- t(res.opta)%*%t(X)%*%Y%*%res.optb
    # update
    na <- res.opta
    nb <- res.optb
    testval <- abs(obj.new - obj.old)
    obj.old <- obj.new
    iter <- iter + 1
    if (obj.old == 0 & obj.new == 0) break
    else if (testval < 1e-4) break
  }
  return(list("call" = call, "res.opta" = res.opta, "res.optb" = res.optb))
}
