#' Structured Sparse Co-Inercia Analysis
#'
#' This function conduct structured sparse CIA for a given tuning parameter value.
#' @param X  n-by-p transformed data matrix of \code{X}, samples are rows and columns are features.
#' @param Y  n-by-q transformeddata matrix of \code{Y}, samples are rows and columns are features.
#' @param na  p-by-1 vector, initial values of loading vector of variables in \code{X}.
#' @param nb  q-by-1 vector, initial values of loading vector of variables in \code{Y}.
#' @param Sx \eqn{Qx^{-1/2} E_x \Gamma_x^{1/2}} where \eqn{E_x} and \eqn{\Gamma_x^{1/2}} are
#' eigen vectors and eigen values of the Laplacian matrix of \code{X}. This is the output of \code{getNetMatrix}.
#' @param Sy \eqn{Qy^{-1/2} E_y \Gamma_y^{1/2}} where \eqn{E_y} and \eqn{\Gamma_y^{1/2}} are
#' eigen vectors and eigen values of the Laplacian matrix of \code{Y}. This is the output of \code{getNetMatrix}.
#' @param lambdas 4-by-1 vector, the first and third values are sparsity penalty parameter for
#' variables in \code{X} and \code{Y} each and the second and fourth values are network
#' penalty parameters for variables in \code{X} and \code{Y} each.
#' @return This function returns a list object.
#' \item{call}{the matched call.}
#' \item{res.opta}{the estimated loading vector of of transformated matrix of \code{X}.}
#' \item{res.optb}{the estimated loading vector of of transformated matrix of \code{Y}.}
#' @seealso see also \code{\link{oCIA}}, \code{\link{getNetMatrix}}, \code{\link{ssCIA.cv}}.
#' @examples
#' data("demoData"); attach(demoData)
#' X <- setX[1:n,]; nX <- sweep(X, 2, apply(X, 2, mean))
#' Y <- setY[1:n,]; nY <- sweep(Y, 2, apply(Y, 2, mean))
#' result.cia <- oCIA(nX, nY, D, Qx, Qy)
#' netmats <- getNetMatrix(p, q, eX, eY, Qx, Qy)
#' tLx <- netmats$Sx # tilde{L}_x
#' tLy <- netmats$Sy # tilde{L}_y
#' result.sscia <- fitsscia(nX, nY, result.cia$res.opta[,1], result.cia$res.optb[,1], tLx, tLy, c(6, 1, 6, 0.5))
#' @export
fitsscia <- function(X, Y, na, nb, Sx, Sy, lambdas, maxiter=100) {
  call <- match.call()
  p <- dim(X)[2]
  q <- dim(Y)[2]
  iter <- 1
  obj.old <- 1
  for (iter in 1:maxiter) {
    # fitting - a
    Astar <- rbind(diag(1, nrow = p), sqrt(lambdas[2]) * t(Sx))
    bstar <- rbind(t(X) %*% Y %*% nb, matrix(0, p, 1))
    res.a <- glmnet::glmnet(x = Astar, y = bstar, 
                            intercept = FALSE, 
                            alpha = 1,
                            lambda = lambdas[1])
    res.opta <- as.matrix(res.a$beta)
    if (as.numeric(sqrt(t(res.opta) %*% res.opta)) != 0) {
      res.opta <- res.opta / as.numeric(sqrt(t(res.opta) %*% res.opta))
    } else {
      res.opta <- matrix(0, p, 1)
    }
    # fitting - b
    Bstar <- rbind(diag(1, nrow = q), sqrt(lambdas[4]) * t(Sy))
    astar <- rbind(t(Y) %*% X %*% na, matrix(0, q, 1))
    res.b <- glmnet::glmnet(x = Bstar, y = astar, 
                            intercept = FALSE, 
                            alpha = 1,
                            lambda = lambdas[3])
    res.optb <- as.matrix(res.b$beta)
    if (as.numeric(sqrt(t(res.optb) %*% res.optb)) != 0) {
      res.optb <- res.optb / as.numeric(sqrt(t(res.optb) %*% res.optb))
    } else {
      res.optb <- matrix(0, q, 1)
    }
    obj.new <- t(res.opta)%*%t(X)%*%Y%*%res.optb
    # update
    na <- res.opta
    nb <- res.optb
    testval <- abs(obj.new - obj.old) / obj.old
    obj.old <- obj.new
    iter <- iter + 1
    if (obj.old == 0 & obj.new == 0) break
    else if (testval < 1e-3) break
  }
  return(list("call"= call, "res.opta" = res.opta, "res.optb" = res.optb))
}
