#' Generate Co-inertia Analysis Figure
#'
#' This function generate an object that is used as an input for the function \code{\link{plot.pcia}} that generates
#'  CIA plots.
#' @param respcia an object of class \code{pcia}, result output of functions \code{\link{fitscia}},\code{\link{fitsscia}},
#' \code{\link{sCIA.cv}}, and \code{\link{ssCIA.cv}}.
#' @param namesX  variable name of data \code{X}.
#' @param namesY  variable name of data \code{Y}.
#' @param samplename list of sample names.
#' @param class class information of samples.
#' @param X  n-by-p data matrix, samples are rows and columns are features.
#' @param Y  n-by-q data matrix, samples are rows and columns are features.
#' @param D  n-by-n diagonal weight matrix for samples.
#' @param Qx  p-by-p diagonal weight matrix for variables of X.
#' @param Qy  q-by-q diagonal weight matrix for variables of Y.
#' @return This function returns required coordinate informations and label informations used for generating CIA plots. 
#' @seealso  see also plot.pcia
#' @examples
#' library(pCIA)
#' data(list="NCI60", package="made4")
#' X <- t(NCI60$Ross)
#' Y <- t(NCI60$Affy)
#' D <- diag(dim(X)[1]); Qx <- diag(apply(abs(X), 2, sum) / sum(abs(X))); Qy <- diag(apply(abs(Y), 2, sum) / sum(abs(Y)))
#' nX <- sweep(X, 2, apply(X, 2, mean)); nY <- sweep(Y, 2, apply(Y, 2, mean))
#' result.cia <- oCIA(nX, nY, D, Qx, Qy)
#' pinput <- genpInput(result.cia, NCI60$Annot[,2], NCI60$Annot[,4], NCI60$classes[,1], NCI60$classes[,2], nX, nY, D, Qx, Qy)

#' @export
genpInput = function(respcia, namesX, namesY, samplename, class, X, Y, D, Qx, Qy){
  nX = sweep(X, 2, apply(X, 2, mean))
  nY = sweep(Y, 2, apply(Y, 2, mean))
  nX = sqrt(D)%*%nX%*%sqrt(Qx)
  nY = sqrt(D)%*%nY%*%sqrt(Qy)

  mX = mY = NULL
  for (i in 1:2){
  lX = nX%*%respcia$res.opta[,i]
  denX = sqrt(diag(t(lX)%*%lX))
  tmpx = lX / matrix(denX, dim(lX)[1], dim(lX)[2], byrow=T)
  mX = cbind(mX, tmpx)

  lY = nY%*%respcia$res.optb[,i]
  denY = sqrt(diag(t(lY)%*%lY))
  tmpy = lY / matrix(denY, dim(lY)[1], dim(lY)[2], byrow = T)
  mY = cbind(mY, tmpy)
  }

  Xcoord = t(nX)%*%nY%*%respcia$res.optb[,1:2]
  Ycoord = t(nY)%*%nX%*%respcia$res.opta[,1:2]

  rownames(Xcoord) = namesX; rownames(Ycoord) = namesY
  rownames(mX) = rownames(mY) = samplename
  colnames(Xcoord) = colnames(Ycoord) = colnames(mX) = colnames(mY) =c("lv1", "lv2")
  mX = data.matrix(mX); mY = data.matrix(mY); Xcoord=data.matrix(Xcoord); Ycoord=data.frame(Ycoord)

  input_pCIAplot = list(mX=mX, mY=mY, co = Xcoord, li=Ycoord, call=respcia$call[1:3], namesX=namesX, namesY=namesY, class=class)
  class(input_pCIAplot) = "pcia"
  return(input_pCIAplot)
}
