#' Generating Laplacian Matrix and its Eigendecompistion Results
#'
#' This function calculate Laplacian matrix and computes the Eigendecompistion of the Laplacian matrix to
#' generate input matrix used in the ssCIA.
#'
#' @param p  number of variables in the data \code{X}.
#' @param q  number of variables in the data \code{Y}.
#' @param eX matrix that has two columns. Each rows contains location of variabes
#' that are connected to each other in the data \code{X}.
#' @param eY matrix that has two columns. Each rows contains location of variabes
#' that are connected to each other in the data \code{Y}.
#' @param Qx  p-by-p diagonal weight matrix for variables of \code{X}.
#' @param Qy  q-by-q diagonal weight matrix for variables of \code{Y}.
#' @return This function returns a list object.
#'
#' \item{Sx}{\eqn{Qx^{-1/2} E_x \Gamma_x^{1/2}} where \eqn{E_x} and \eqn{\Gamma_x^{1/2}} are
#' eigen vectors and eigen values of the Laplacian matrix of \code{X}. This is the output of \code{getNetMatrix}.}
#' \item{Sy}{\eqn{Qy^{-1/2} E_y \Gamma_y^{1/2}} where \eqn{E_y} and \eqn{\Gamma_y^{1/2}} are
#' eigen vectors and eigen values of the Laplacian matrix of \code{Y}. This is the output of \code{getNetMatrix}.}

#' @examples
#' data(list="NCI60", package="made4")
#' X <- t(NCI60$Ross); Y <- t(NCI60$Affy)
#' Qx <- diag(apply(abs(X), 2, sum) / sum(abs(X)))
#' Qy <- diag(apply(abs(Y), 2, sum) / sum(abs(Y)))
#' nX <- sweep(X, 2, apply(X, 2, mean))
#' nY <- sweep(Y, 2, apply(Y, 2, mean))
#' data(NCI60path)
#' netmats <- getNetMatrix(dim(nX)[2], dim(nY)[2], NCI60path, NCI60path, Qx, Qy)

#' @export
getNetMatrix <- function(p, q, eX, eY, Qx, Qy){
isQx = diag(1/sqrt(diag(Qx)))
isQy = diag(1/sqrt(diag(Qy)))
gx <- igraph::make_undirected_graph(eX, n=p)
AdjMx <- igraph::as_adjacency_matrix(gx, sparse=FALSE)
rownames(AdjMx) <- colnames(AdjMx) <- c(1:p)

gy <- igraph::make_undirected_graph(eY, n=q)
AdjMy <- igraph::as_adjacency_matrix(gy, sparse=FALSE)
rownames(AdjMy) <- colnames(AdjMy) <- c(1:q)

diagLx <- apply(AdjMx,1,sum)
diagLy <- apply(AdjMy,1,sum)

nLx <- diag(1,p,p) - diag(1/sqrt(diagLx))%*%AdjMx%*%diag(1/sqrt(diagLx))
nLx[is.nan(nLx)] <- 0
nLy <- diag(1,q,q) - diag(1/sqrt(diagLy))%*%AdjMy%*%diag(1/sqrt(diagLy))
nLy[is.nan(nLy)] <- 0

eigLx <- eigen(as.matrix(nearPD(nLx)$mat))
Sx <- eigLx$vectors%*%diag(sqrt(eigLx$values))
Sx <- isQx%*%Sx

eigLy <- eigen(as.matrix(nearPD(nLy)$mat))
Sy <- eigLy$vectors%*%diag(sqrt(eigLy$values))
Sy <- isQy%*%Sy

return(list("Sx" = Sx, "Sy" = Sy))
}
