#' Co-Inercia Analysis (CIA)
#'
#' This function conduct CIA.
#' @param X  n-by-p data matrix, samples are rows and columns are features.
#' @param Y  n-by-q data matrix, samples are rows and columns are features.
#' @param D  n-by-n diagonal weight matrix for samples.
#' @param Qx  p-by-p diagonal weight matrix for variables of X.
#' @param Qy  q-by-q diagonal weight matrix for variables of Y.
#' @return This function returns a list object.
#' \item{call}{the matched call.}
#' \item{res.opta}{estimated loading vectors of of the transformated matrix \eqn{D^{1/2} X Qx^{1/2}}.}
#' \item{res.optb}{estimated loading vectors of of the transformated matrix \eqn{D^{1/2} Y Qy^{1/2}}.}
#' \item{res.optu}{estimated loading vectors of of the matrix \code{X}.}
#' \item{res.optv}{estimated loading vectors of of the matrix \code{Y}.}
#' \item{ecoi}{estimated co-inertia values}
#' @examples
#' data("demoData"); attach(demoData)
#' X <- setX[1:n,]; nX <- sweep(X, 2, apply(X, 2, mean))
#' Y <- setY[1:n,]; nY <- sweep(Y, 2, apply(Y, 2, mean))
#' result.cia <- oCIA(nX, nY, D, Qx, Qy)
#' @export
oCIA <- function(X, Y, D, Qx, Qy){
  call <- match.call()
  
  eigX <- eigen(Qx)
  sQx <-eigX$vectors %*% diag(sqrt(eigX$values))%*%t(eigX$vectors)
  isQx <-eigX$vectors %*% diag(1/sqrt(eigX$values))%*%t(eigX$vectors)

  eigY <-eigen(Qy)
  sQy <- eigY$vectors %*% diag(sqrt(eigY$values))%*%t(eigY$vectors)
  isQy <- eigY$vectors %*% diag(1/sqrt(eigY$values))%*%t(eigY$vectors)

  w <- sQx%*%t(X)%*%D%*%Y%*%sQy

  svdw <- svd(w)

  ea <- svdw$u
  eb <- svdw$v
  eu <- isQx%*%ea
  ev <- isQy%*%eb
  ecoi <- (svdw$d)^2
  
  return(list("call" = call, "res.opta"=ea, "res.optb"=eb, "res.optu"=eu, "res.optv"=ev, "ecoi"=ecoi))
}
