#' Sparse Co-Inercia Analysis  with Cross Validation for Chosing Optimal Tuning Parameters
#'
#' This function can be used to automatically select tuning parameters for sparse CIA.
#' @param X  n-by-p data matrix, samples are rows and columns are features.
#' @param Y  n-by-q data matrix, samples are rows and columns are features.
#' @param D  n-by-n diagonal weight matrix for samples.
#' @param Qx  p-by-p diagonal weight matrix for variables of X.
#' @param Qy  q-by-q diagonal weight matrix for variables of Y.
#' @param K  number of loading pairs to be estimated.
#' @param TRUEval a list containing true loading vectors of \code{X} and \code{Y} in the simulation.
#' @param nfold  number of subgroups that data is separated into for cross validation procedure.
#' @param ngrid  number of possible tuning paramter values that will be searched.
#' @param lambdarange a vector contains four numbers. First and the second values are the minimum
#' and maximum value of ranges for tuniing parameters for \code{X}, the third and fourth are the
#' minimum and maximum value of ranges for tuniing parameters for \code{Y}.
#' @return This function returns a list object. If this is used for a simulation and
#' \code{TRUEval} is used, the output list contains \code{CVobjs}, \code{OptTaus},
#' and \code{resa}, \code{resb}, \code{res.mea}, \code{res.num}. If \code{TRUEval} is set as default,
#' \code{NULL}, then the output list contains \code{Cvobjs}, \code{OptTaus},
#' and \code{resa}, and \code{resb}.
#' \item{call}{the matched call.}
#' \item{CVobjs}{cross validation objective values for all combination of tuning parameters.}
#' \item{OptTaus}{the chosen optimal tuning parameter pairs.}
#' \item{res.opta}{estimated loading vectors of of the transformated matrix \eqn{D^{1/2} X Qx^{1/2}}.}
#' \item{res.optb}{estimated loading vectors of of the transformated matrix \eqn{D^{1/2} Y Qy^{1/2}}.}
#' \item{res.mea}{feature selction performance measures, sensitivity, specificity, MCC, and angle.}
#' \item{res.num}{count values, true positives, true negatives, false positives, and false negatives.}
#' @seealso  see also \code{\link{fitscia}}.
#' @examples
#' data("demoData"); attach(demoData)
#' X <- setX[1:n,]; nX <- sweep(X, 2, apply(X, 2, mean))
#' Y <- setY[1:n,]; nY <- sweep(Y, 2, apply(Y, 2, mean))
#' result.scia.cv <- sCIA.cv(nX, nY, D, Qx, Qy, K=2, nfold=5, ngrid=10, flag=FALSE)
#' @export
sCIA.cv <- function(X, Y, D, Qx, Qy, K=1, TRUEval = NULL, nfold=5, ngrid=10, lambdarange = NULL, flag=TRUE){
  call <- match.call()
  n <- dim(X)[1]
  p <- dim(X)[2]
  q <- dim(Y)[2]
  if (dim(X)[1] != dim(Y)[1])
    stop("Error: different sample sizes")
  if (!Matrix::isDiagonal(D))
    stop("Error: D is required to be diagonal")  
  if (!Matrix::isDiagonal(Qx))
    stop("Error: Qx is required to be diagonal")
  if (!Matrix::isDiagonal(Qy))
    stop("Error: Qy is required to be diagonal")
  if (dim(X)[2] != dim(Qx)[1])
    stop("Error: not one-to-one weights for variables of X")
  if (dim(Y)[2] != dim(Qy)[1])
    stop("Error: not one-to-one weights for variables of Y")

  sQx <- sqrt(Qx)
  sQy <- sqrt(Qy)
  nX <- sweep(X, 2, apply(X, 2, mean))
  nY <- sweep(Y, 2, apply(Y, 2, mean))
  nX <- sqrt(D)%*%nX%*%sQx
  nY <- sqrt(D)%*%nY%*%sQy
  # calculate starting point - use nonsparse cia value
  nonsparse.res <- svd(t(nX)%*%nY)
  resa = resb = OptTaus = res.mea = res.num = cvobjs = NULL
  for (l in 1:K){
    nonsparse.res.a <- as.matrix(nonsparse.res$u[,l])
    nonsparse.res.b <- as.matrix(nonsparse.res$v[,l])
    # for CV, generate the id of data to split the data into nfold subgroups
    foldid <- sample(rep(seq_len(nfold), length.out=n))
    tmp.a <- glmnet::glmnet(x=diag(1,p), 
                            y=t(nX)%*%nY%*%nonsparse.res.b, 
                            intercept=FALSE, 
                            alpha=1)
    tmp.b <- glmnet::glmnet(x=diag(1,q), 
                            y=t(nY)%*%nX%*%nonsparse.res.a, 
                            intercept=FALSE, 
                            alpha=1)
    if (is.null(lambdarange)){
    l1 <- seq(min(tmp.a$lambda),max(tmp.a$lambda), length=ngrid)
    l2 <- seq(min(tmp.b$lambda),max(tmp.b$lambda), length=ngrid)
    }else{
    l1 <- seq(lambdarange[1],lambdarange[2], length=ngrid)
    l2 <- seq(lambdarange[3],lambdarange[4], length=ngrid)
    }
    gridpt <- cbind(rep(l1,each=length(l2)), rep(l2, length(l1)))
    cvobj <- rep(0,dim(gridpt)[1])
    for (j in 1:dim(gridpt)[1]){
      # reset cv criteria values
      avgLtest = avgLtrain = avgLtest2 = avgLtrain2 = 0
      for (k in 1:nfold){
        # split the data into test and training set
        xtest <- nX[foldid == k,]
        ztest <- nY[foldid == k,]
        xtrain <- nX[foldid != k,]
        ztrain <- nY[foldid != k,]
        # fit training data
        res.train <- fitscia(xtrain, ztrain, nonsparse.res.a, nonsparse.res.b, gridpt[j,])
        res.traina <- res.train$res.opta
        res.trainb <- res.train$res.optb
        # calculate objectives for test and training data
        Ltest = (t(res.traina)%*%t(xtest)%*%ztest%*%res.trainb)^2
        avgLtest = avgLtest + Ltest
      }
      # calculate cv objective values
      cvobj[j] = avgLtest/nfold
      if (flag==TRUE){
        print(paste("For ", j,"-th grid point, cv objective value is ",cvobj[j], sep=""))
      }
    }
    # choose optimal tuning paramter based on cv criteria
    ind <- which(cvobj == max(cvobj))
    optTaus <- gridpt[ind[length(ind)],]
    # fit ssCIA again using optimal tuning parameters
    cvopt.result <- fitscia(nX, nY, 
                            nonsparse.res.a, nonsparse.res.b, 
                            optTaus)
    res.opta <- cvopt.result$res.opta
    res.optb <- cvopt.result$res.optb
    # store final result for l-th dimension
    OptTaus <- c(OptTaus, optTaus)
    resa <- cbind(resa, res.opta)
    resb <- cbind(resb, res.optb)
    cvobjs <- rbind(cvobjs, cvobj)
    if (is.null(TRUEval) == FALSE){
      ta <- TRUEval$a
      tb <- TRUEval$b
      res.calmea <- calmeasure(res.opta, res.optb, ta[,l], tb[,l])
      res.mea <- rbind(res.mea, res.calmea$measures)
      res.num <- rbind(res.num, res.calmea$numbers)
    }
    # deflation for getting next coefficient
    nX <- nX %*% (diag(1, p) - res.opta %*% t(res.opta))
    nY <- nY %*% (diag(1, q) - res.optb %*% t(res.optb))
    cat(sprintf("%dth dimension is done\n", l))
  }
  if (is.null(TRUEval) == FALSE){
    return(list("call" = call, "CVobjs" = cvobjs, "OptTaus" = OptTaus, "res.opta" = resa, "res.optb" = resb, "res.mea" = res.mea, "res.num" = res.num))
  }else{
    return(list("call" = call, "CVobjs" = cvobjs, "OptTaus" = OptTaus, "res.opta" = resa, "res.optb" = resb))
  }
}
