#' Structured Sparse Co-Inercia Analysis  with Cross Validation for Chosing Optimal Tuning Parameters
#'
#' This function can be used to automatically select tuning parameters for structured sparse CIA.
#' @param X  n-by-p data matrix, samples are rows and columns are features.
#' @param Y  n-by-q data matrix, samples are rows and columns are features.
#' @param D  n-by-n diagonal weight matrix for samples.
#' @param Qx  p-by-p diagonal weight matrix for variables of \code{X}.
#' @param Qy  q-by-q diagonal weight matrix for variables of \code{Y}.
#' @param Sx \eqn{Qx^{-1/2} E_x \Gamma_x^{1/2}} where \eqn{E_x} and \eqn{\Gamma_x^{1/2}} are
#' eigen vectors and eigen values of the Laplacian matrix of \code{X}. This is the output of \code{getNetMatrix}.
#' @param Sy \eqn{Qy^{-1/2} E_y \Gamma_y^{1/2}} where \eqn{E_y} and \eqn{\Gamma_y^{1/2}} are
#' eigen vectors and eigen values of the Laplacian matrix of \code{Y}. This is the output of \code{getNetMatrix}.
#' @param K  number of loading pairs to be estimated.
#' @param TRUEval a list containing true loading vectors of \code{X} and \code{Y} in the simulation.
#' @param nfold  number of subgroups that data is separated into for cross validation procedure.
#' @param ngrid  number of possible tuning paramter values that will be searched.
#' @param lambdarange a vector contains four numbers he range of sparsity penalty parameters.
#' \code{genGridPT} function calculates grid points based on this ranage. Default values are \code{NULL},
#' in this case \code{genGridPT} function calculate range of tuning parameters from the result of lasso regression of glm package.
#' Users can set the range of tuning parameters by putting desirable values. First and the second values
#' are the minimum and maximum value of ranges for tuning parameters for \code{X}, the third and fourth area
#' the minimum and maximum value of ranges for tuning parameters for \code{Y}.
#' @param lambda2range a vector contains four numbers the range of network penalty parameters.
#' \code{genGridPT} function calculates grid points based on this ranage. Default values are \code{NULL},
#' in this case \code{genGridPT} function use three values (0.1, 0.5, 1) as a grid point.
#' Users can set the range of tuning parameters by putting desirable values. First and the second values
#' are the minimum and maximum value of ranges for tuning parameters for \code{X}, the third and fourth area
#' the minimum and maximum value of ranges for tuning parameters for \code{Y}.
#' @return This function returns a list object. If this is used for a simulation and
#' \code{TRUEval} is used, the output list contains \code{CVobjs}, \code{OptTaus},
#' and \code{resa}, \code{resb}, \code{res.mea}, \code{res.num}. If \code{TRUEval} is set as default,
#' \code{NULL}, then the output list contains \code{Cvobjs}, \code{OptTaus},
#' and \code{resa}, and \code{resb}.
#' \item{call}{the matched call.}
#' \item{CVobjs}{cross validation objective values for all combination of tuning parameters.}
#' \item{OptTaus}{the chosen optimal tuning parameter pairs.}
#' \item{resa}{estimated loading vectors of of the transformated matrix \eqn{D^{1/2} X Qx^{1/2}}.}
#' \item{resb}{estimated loading vectors of of the transformated matrix \eqn{D^{1/2} Y Qy^{1/2}}.}
#' \item{res.mea}{feature selction performance measures, sensitivity, specificity, MCC, and angle.}
#' \item{res.num}{count values, true positives, true negatives, false positives, and false negatives.}
#' @seealso see also \code{\link{oCIA}}, \code{\link{getNetMatrix}}, \code{\link{fitsscia}}.
#' @examples
#' data("demoData"); attach(demoData)
#' X <- setX[1:n,]; nX <- sweep(X, 2, apply(X, 2, mean))
#' Y <- setY[1:n,]; nY <- sweep(Y, 2, apply(Y, 2, mean))
#' netmats <- getNetMatrix(p, q, eX, eY, Qx, Qy)
#' tLx <- netmats$Sx # tilde{L}_x
#' tLy <- netmats$Sy # tilde{L}_y
#' result.sscia.cv <- ssCIA.cv(nX, nY, D, Qx, Qy, tLx, tLy, K=2, nfold=5, ngrid=c(5, 5, 5 ,5), flag=FALSE)
#' @export
ssCIA.cv <- function(X, Y, D, Qx, Qy, Sx, Sy, K=1, nfold=5, TRUEval = NULL,
                    ngrid=c(10, 3, 10 ,3), lambdarange=NULL, lambda2range=NULL, flag=TRUE){
  call <- match.call()
  n <- dim(X)[1]
  p <- dim(X)[2]
  q <- dim(Y)[2]
  if (dim(X)[1] != dim(Y)[1])
    stop("Error: different sample sizes")
  if (!Matrix::isDiagonal(D))
    stop("Error: D is required to be diagonal")  
  if (!Matrix::isDiagonal(Qx))
    stop("Error: Qx is required to be diagonal")
  if (!Matrix::isDiagonal(Qy))
    stop("Error: Qy is required to be diagonal")
  if (dim(X)[2] != dim(Qx)[1])
    stop("Error: not one-to-one weights for variables of X")
  if (dim(Y)[2] != dim(Qy)[1])
    stop("Error: not one-to-one weights for variables of Y")
  sQx <- sqrt(Qx)
  sQy <- sqrt(Qy)
  nX <- sweep(X, 2, apply(X, 2, mean))
  nY <- sweep(Y, 2, apply(Y, 2, mean))
  nX <- sqrt(D)%*%nX%*%sQx
  nY <- sqrt(D)%*%nY%*%sQy
  # calculate starting point - use nonsparse cia value
  nonsparse.res <- svd(t(nX)%*%nY)
  resa = resb = OptTaus = res.mea = res.num = cvobjs = NULL
  for (l in 1:K){
    nonsparse.res.a <- as.matrix(nonsparse.res$u[,l])
    nonsparse.res.b <- as.matrix(nonsparse.res$v[,l])
    # for CV, generate the id of data to split the data into nfold subgroups
    foldid <- sample(rep(seq_len(nfold), length.out=n))
    gridpt <- genGridPT(nX, nY, 
                        nonsparse.res.a, nonsparse.res.b, 
                        Sx, Sy, 
                        ngrid, 
                        lambdarange, lambda2range)
    cvobj <- rep(0,dim(gridpt)[1])
    for (j in 1:dim(gridpt)[1]){
      # reset cv criteria values
      avgLtest = avgLtrain = avgLtest2 = avgLtrain2 = 0
      for (k in 1:nfold){
        # split the data into test and training set
        xtest <- nX[foldid == k,]
        ztest <- nY[foldid == k,]
        xtrain <- nX[foldid != k,]
        ztrain <- nY[foldid != k,]
        # fit training data
        res.train <- fitsscia(xtrain, ztrain, 
                              nonsparse.res.a, nonsparse.res.b,
                              Sx, Sy, 
                              gridpt[j,])
        res.traina <- res.train$res.opta
        res.trainb <- res.train$res.optb
        # calculate objectives for test and training data
        Ltest <- (t(res.traina)%*%t(xtest)%*%ztest%*%res.trainb)^2
        avgLtest <- avgLtest + Ltest
      }
      # calculate cv objective values
      cvobj[j] <- avgLtest/nfold
      if (flag == TRUE){
        print(paste("For ", j,"-th grid point, cv objective value is ",cvobj[j], sep=""))
      }
    }
    # choose optimal tuning paramter based on cv criteria
    ind <- which(cvobj == max(cvobj))
    optTaus <- gridpt[ind[length(ind)],]
    # fit ssCIA again using optimal tuning parameters
    cvopt.result <- fitsscia(nX, nY, 
                             nonsparse.res.a, nonsparse.res.b,
                             Sx, Sy,
                             optTaus)
    res.opta <- cvopt.result$res.opta
    res.optb <- cvopt.result$res.optb
    # store final result for l-th dimension
    OptTaus <- c(OptTaus, optTaus)
    resa <- cbind(resa, res.opta)
    resb <- cbind(resb, res.optb)
    cvobjs <- rbind(cvobjs, cvobj)
    if (is.null(TRUEval) == FALSE){
      ta <- TRUEval$a
      tb <- TRUEval$b
      res.calmea <- calmeasure(res.opta, res.optb, ta[,l], tb[,l])
      res.mea <- rbind(res.mea, res.calmea$measures)
      res.num <- rbind(res.num, res.calmea$numbers)
    }
    # deflation for getting next coefficient
    nX <- nX %*% (diag(1, p) - res.opta %*% t(res.opta))
    nY <- nY %*% (diag(1, q) - res.optb %*% t(res.optb))
    cat(sprintf("%dth dimension is done\n", l))
  }
  if (is.null(TRUEval) == FALSE){
    return(list("call" = call, "CVobjs" = cvobjs, "OptTaus" = OptTaus, "res.opta" = resa, "res.optb" = resb, "res.mea" = res.mea, "res.num" = res.num))
  }else{
    return(list("call" = call, "CVobjs" = cvobjs, "OptTaus" = OptTaus, "res.opta" = resa, "res.optb" = resb))
  }
}
